/********************************************/
/*  Modbus/TCP protocol converter head file */
/*  For 8430/8830/8431/8831                 */
/*  Version: 1.0.2                          */
/*  [25,Oct,2002] by Kevin                  */
/********************************************/
/*
Version 1.3.7 [4,Jan,2006] by Kevin
    Remove un-usefull comments in fucntion ModbusRTU_Master2Slave
    and ModbusASCII_Master2Slave
    
Version 1.3.7 [29,Sep,2005] by Kevin
    Change ModbusMaster2Slave:
        Rename to ModbusRTU_Master2Slave,
        Add a lTimeout parameter.
        
    Add ModbusASCII_Master2Slave 
---------------------------------------------------------------------
Version 1.2.0 [26,Aug,2005] by Kevin
    Add functions for Modbus/TCP master 
        (1) ModbusTCP_Init
        (2) ModbusTCP_Master2Slave
        (3) ModbusTCP_Close
---------------------------------------------------------------------
Version 1.1.7 [24,Dec,2004] by Kevin
    Rename CheckLEDMenu to SMMI_Offset_CheckIn
    
    Add SMMI_Net_CheckIn
---------------------------------------------------------------------
Version 1.1.6 [15,Jun,2004] by Kevin
    Change declaration                          
        1. _MaxModule: constant ==> int iMax_Register_Count
        2. iMemory_DI, iMemory_DO,              
           iMemory_AI, iMemory_AO ==> to pointer
---------------------------------------------------------------------
Version 1.1.5 [23,Sep,2002] by Kevin
    Add function CheckLEDMenu. 
---------------------------------------------------------------------
Version 1.1.4 [14,Jun,2003] by Kevin  
    Add comment to ModbusMaster2Slave
---------------------------------------------------------------------
Version 1.1.3 [26,Mar,2003] by Kevin
    Add variable iModbusUpLinkPort.  
---------------------------------------------------------------------
Version 1.1.2 [08,Mar,2003] by Kevin
    Add function CheckModbusRequest.            
---------------------------------------------------------------------
Version 1.1.0 [11,Nov,2002] by Kevin
    Add function ModbusMaster2Slave.
*/



//For debuging
extern int iDebugPort;
extern unsigned long lDebugBaudrate;
    // Set these 2 variables before calling InitModbus().
    // That lets modbus library shows Modbus/TCP requese/response messages.


extern int iMax_Register_Count; // Maximum register count of iMemory_DI, 
                                //                           iMemory_DO,
                                //                           iMemory_AI and
                                //                           iMemory_AO.
                                // The default register count is 500. 
                                // The maximum channel for one DI or DO modules 
                                // is 32, and for AI and AO modules is 8. 
                                // Thus the I/O modules could use
                                // 32(channel)*8(slot) DI registers,
                                // 32(channel)*8(slot) DO registers, 
                                // 8(channel)*8(slot) AI registers or
                                // 8(channel)*8(slot) AO registers.
                                //
                                // Users can use other free registers to store
                                // extra information. If the registers are not 
                                // enough, you can change iMax_Module_Count to 
                                // enlarge the register count. 
                                // 
                                // Usage:
                                //   iMax_Register_Count=1000;
                                //   iRet=InitModbus();
                                // 
                                // Note:  iMax_Register_Count should be
                                //        >= I/O module channels (mentioned above)
                                //        <= 32767
                               
//Memory base address of every slot
extern unsigned int iMemoryAddr_DI[8];
extern unsigned int iMemoryAddr_DO[8];
extern unsigned int iMemoryAddr_AI[8];
extern unsigned int iMemoryAddr_AO[8];

//I/O points of every slot
extern unsigned int iMemoryNum_DI[8];   
extern unsigned int iMemoryNum_DO[8];
extern unsigned int iMemoryNum_AI[8];
extern unsigned int iMemoryNum_AO[8];

//The I/O values 
extern unsigned char* iMemory_DI;   // One digital module has 32 points maximum
extern unsigned char* iMemory_DO;   // One digital module has 32 points maximum
extern          int* iMemory_AI;    // One analog module has 8 points maximum
extern          int* iMemory_AO;    // One analog module has 8 points maximum

extern int iTotalSlot;
extern int iScanSlot;

extern int iModbusUpLinkPort;

extern int iDINum,iDONum,iAINum,iAONum; // Total DI,DO,AI,AO numbers.
    /*
    To put user-defined registers to the momory,
    the begining offset index is iDINum,iDONum,iAINum,iAONum.
    For example:
        iMemory_DI[iDINum]=1;       // the 1st user-defined DI register
        iMemory_DI[iDINum+1]=0;     // the 2nd user-defined DI register
        iMemory_DO[iDONum]=1;       // the 1st user-defined DO register
        iMemory_AI[iAINum+5]=1234;  // the 6th user-defined AI register
        iMemory_AO[iAONum+2]=4567;  // the 3th user-defined AO register
    */

extern int iMenu_Current_Level_Net;
extern int iMenu_Current_Level_Offset;

//===============================================//
//          Prototype of functions               //
//===============================================//
void UpdateIOModule(void);

int InitModbus(void);
    /*
    Scan the hardware and reads settings from EEPROM.
    If doesn't find the config file, 
    this function will use the default settings.
    
    0:  Success ==> NoError
    -1: Memory allocate error.
    */
    

//==================================================//
//      Prototype of I/O module access functions    //
//==================================================//
void ScanIOModule(int iSlot);
    // iSlot: scan which slot now?

    
//======== Configuration commands ==========//
//  Modbus Utility send commands to         //
//  configure Modbus/TCP controllers.       //
//==========================================//  
int Configuration(unsigned char *Command,unsigned char *Result);
    // 0:  Success ==> NoError
    // -1: command doesn't support
    
    
void CheckModbusRequest(int iPort);
    /*
    iPort: Is any Modbus/RTU request from COM port ?
    
    The iPort must be the global variable,iModbusUpLinkPort.
    So, when you call the function in UserLoopFun (in User.c),
    you must call:
         CheckModbusRequest(iModbusUpLinkPort);
    
    The default COM port is COM1 and buadrate is 115200.
    If you want to change the Modbus/RTU slave port,
    you should add 3 codes to UserInit (in User.c):
    */
    //  //======= Begin Modbus Kernel =======
    //  int iRet;
    //  iRet=InitModbus();
    //  if(iRet!=0)
    //  {
    //      // Initial Modbus library error.
    //  }
    //  else
    //  {
    //      // Initial Modbus library ok.
    //  }
    //  iModbusUpLinkPort=3;                    // <=== add this code
    //  SetBaudrate(iModbusUpLinkPort,115200);  // <=== add this code 
    //  SetDataFormat(iModbusUpLinkPort,8,0,1); // <=== add this code
    //                                          // data bit = 8
    //                                          // parity bit = disable
    //                                          // stop bit = 1
    //  //======= End Modbus Kernel =======
    
#define CheckLEDMenu SMMI_Offset_CheckIn
int SMMI_Offset_CheckIn(void);
    /*
    Modbus libarary provide a LED menu from version 1.2.0.
    The LED menu can be used to adjust AI offset.
    Please refer to CD:\Napdos\Modbus\8000e\document\LED_Menu.pdf
    for the detail information.
    */
    

int SMMI_Net_CheckIn(void);


//==================================================//
//                                                  //
//      Modbus master functions                     //
//        for Modbus/RTU                            //
//        for Modbus/ASCII                          //
//        for Modbus/TCP                            // 
//==================================================//
int ModbusRTU_Master2Slave(int iPort,unsigned char cNetID, unsigned char cFunction,
    int iControllerMemoryBaseAddress, int iDeviceMemoryBaseAddress,int iIOCount,unsigned long lTime);
    /*
    This functions can send Modbus/RTU request to Modbus/Slave devices.
    1. You just only change internal registers (iMemory_DI, iMemory_DO,
       iMemory_AI, iMemory_AO) then call this function to send out request to turn on coils
       or write analog registers.
    2. Sends out  
    
    iPort: 1 ==> COM1 
           2 ==> COM2 
           ...
           Sends request to which COM port?
    cNetID: 0~0xFF, the NetID (Station Number) of destination Modbus/Slave device.
    cFunction:  1 ==> Reads  several DO from device, then puts to iMemory_DO.
                2 ==> Reads  several DI from device, then puts to iMemory_DI.
                3 ==> Reads  several AO from device, then puts to iMemory_AO.
                4 ==> Reads  several AI from device, then puts to iMemory_AI.
                5 ==> Writes     one DO from iMemory_DO to device. 
                6 ==> Writes     one AO from iMemory_AO to device.
               15 ==> Writes several DO from iMemory_DO to device.
               16 ==> Writes several AO from iMemory_AO to device.
               
    iControllerMemoryBaseAddress: internal register base address that you want to deal.
    iDeviceMemoryBaseAddress: register base address of device that you want to deal.
    iIOCount: count of coils or registers that you want to deal.
    
    Return value:
         0: No Error
        -1: CRC error
        -2: Receiving response timeout (received 0 bytes)
        -3: Receiving response timeout (received less than 5 bytes)
        -4: COM port function WaitTransmitOver error
        >0: Exception code from the Modbus slave device.
    */
    
int ModbusASCII_Master2Slave(int iPort,unsigned char cNetID, unsigned char cFunction,
    int iControllerMemoryBaseAddress, int iDeviceMemoryBaseAddress,int iIOCount,unsigned long lTime);
    /*
    This functions can send Modbus/RTU request to Modbus/Slave devices.
    1. You just only change internal registers (iMemory_DI, iMemory_DO,
       iMemory_AI, iMemory_AO) then call this function to send out request to turn on coils
       or write analog registers.
    2. Sends out  
    
    iPort: 1 ==> COM1 
           2 ==> COM2 
           ...
           Sends request to which COM port?
    cNetID: 0~0xFF, the NetID (Station Number) of destination Modbus/Slave device.
    cFunction:  1 ==> Reads  several DO from device, then puts to iMemory_DO.
                2 ==> Reads  several DI from device, then puts to iMemory_DI.
                3 ==> Reads  several AO from device, then puts to iMemory_AO.
                4 ==> Reads  several AI from device, then puts to iMemory_AI.
                5 ==> Writes     one DO from iMemory_DO to device. 
                6 ==> Writes     one AO from iMemory_AO to device.
               15 ==> Writes several DO from iMemory_DO to device.
               16 ==> Writes several AO from iMemory_AO to device.
               
    iControllerMemoryBaseAddress: internal register base address that you want to deal.
    iDeviceMemoryBaseAddress: register base address of device that you want to deal.
    iIOCount: count of coils or registers that you want to deal.
    
    Return value:
         0: No Error
        -1: LRC error
        -2: Receiving response timeout (received 0 bytes)
        -3: Receiving response timeout (received less than 5 bytes)
        -4: COM port function WaitTransmitOver error
        >0: Exception code from the Modbus slave device.
    */    
    
int ModbusTCP_Init(int iIndex,unsigned char *cRemote_IP,int iRemote_TCPPort,
                   unsigned long lConnect_Timeout,unsigned long lReConnect_Time_Interval);
    /*
    Give Modbus/TCP intial values.
    there are 8 connections available. You can assign several 
    connection to one IP address or one connection to each IP address. 
    
    iIndex: 
          0~7. The library can assign totaly 8 connections.
    cRemote_IP: 
          IP address for remote device.
          For example: "192.168.255.100"
    iRemote_TCPPort: 
          TCP port for remote device.
          For example: 502 
    lConnect_Timeout: 
          Timeout to connect to remote device. 
          unit: ms       
    lReConnect_Time_Interval:
          Time interval to reconnect to remote device
          when the
          
    return: 0= ok.
            other = Error.      
    */
    
int ModbusTCP_Master2Slave(int iIndex,unsigned char cNetID, unsigned char cFunction,
                           int iControllerMemoryBaseAddress, int iDeviceMemoryBaseAddress,
                           int iIOCount, unsigned long lTimeout);
    /*
    This functions can send Modbus/TCP request to Modbus/Slave devices.
    1. You just only change internal registers (iMemory_DI, iMemory_DO,
       iMemory_AI, iMemory_AO) then call this function to send out request to turn on coils
       or write analog registers.
    2. Sends out  
    
    iIndex: 
          0~7. The library can assign totaly 8 connections.
               Each connection's configuration is defined by 
               function ModbusTCP_Init.
               
    cNetID: 0~0xFF, the NetID (Station Number) of destination Modbus/Slave device.
    cFunction:  1 ==> Reads  several DO from device, then puts to iMemory_DO.
                2 ==> Reads  several DI from device, then puts to iMemory_DI.
                3 ==> Reads  several AO from device, then puts to iMemory_AO.
                4 ==> Reads  several AI from device, then puts to iMemory_AI.
                5 ==> Writes     one DO from iMemory_DO to device. 
                6 ==> Writes     one AO from iMemory_AO to device.
               15 ==> Writes several DO from iMemory_DO to device.
               16 ==> Writes several AO from iMemory_AO to device.
    iControllerMemoryBaseAddress: internal register base address that you want to deal.
    iDeviceMemoryBaseAddress: register base address of device that you want to deal.
    iIOCount: count of coils or registers that you want to deal.
    lTimeout: 1~65535. unit: ms
              timeout to receive response from Modbus/Slave device.
    return:
         0: No Error.
            Send the command ok, and receive the response ok.
         1: send command ok,
            begin to wait response next time.
         2: send command ok,
            not receive the response yet,
            keep to receive the response next time.
        -1: connect to slave device error,
            try to re-connect next timer interval.
        -2: connection is not builded. Cannot send command. 
        -3: connect to the device ok,
            send command error,
            try to re-connect the device next time interval.
        -4: send command ok,
            wait response timeout,
            try to re-connect the device next time interval.  
        -5: send command ok,
            receive response error.
        -6: the connection is closed or not initialized.  
            You must call ModbusTCP_Init first then this function. 
    */
        
int ModbusTCP_Close(int iIndex);      
    /*
    Close the connection.
    
    iIndex: 
          0~7. The library can assign totaly 8 connections.
    */                